/* filename:    margmh.g
** description: The program maximizes the posterior density of the 
**              marginal log likelihood
**              with respect to the DSGE model parameters
** created:     11/05/01
*/

/********************************************************
** Define lambda indices
*/

llam = "L" $+ ftos(90000+10*lambda,"%*.*lf",5,0);

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

lsamp = "T" $+ lsamp;

/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\" $+ llam $+ lsamp $+ "pm";
lretc = postpath $+ "\\" $+ llam $+ lsamp $+ "pmret";

/** check posterior mode results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para  = para';          /** paraest was saved as row vector in _pm.g **/
npara = rows(para);

open fhretc = ^lretc for read;
_csretcode = readr(fhretc,1);
closeall fhretc;

/********************************************************
* Load multiplication matrix for parameters
********************************************************/
lmult = postpath $+ "\\" $+ llam $+ lsamp $+ "mult";       /** sigmult **/
lhess = postpath $+ "\\" $+ llam $+ lsamp $+ "hess";       /** HHM **/


/** check Hessian results **/
if not(exist(lmult $+ ".dat"));
    "";
    "Please run Hessian finding program first.";
    "";
    end;
endif;

/** open **/
open fhmult = ^lmult for read;
SIGSCALE    = readr( fhmult,npara );
closeall fhmult;

open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/** rank of Hessian (theoretical) **/
SIGPROPdim = sumc(pmaskinv);

/** inverse by Singular Value Decomposition (SVD) **/
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

/** log determinant for nonsingular part **/
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;



opath   = parapath $+ "\\mhrun" $+ mhrun;
chkdir  = chk_dir(opath);

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "MHrun Index      " $mhrun;
    "Scaling of Hess  " cc;
    "";
    "===================================";
    "";

    outmat = ("Parameter"  $~ "Estimate") $|
             ( (para_names') $~ ftocv( para,4,4) );

    $outmat;
        
    {lnpost0, lnpy0} = fcn2(para);

    "";
    "Prior*Likelihood at Mode:" lnpost0;
    "Likelihood at Mode      :" lnpy0;
    "";
    "Press Key to Continue";
    /* k = keyw; */
    "";

endif;      /** _verbose **/



/********************************************************
* Initialize Output files
********************************************************/

statname = { POST LIKE PROP PROPPOST REJECT };

ostat = opath $+ "\\" $+ llam $+ lsamp $+ "st";
opara = opath $+ "\\" $+ llam $+ lsamp $+ "pa";

/** check if append is possible **/
if not(exist(opara $+ ".dat"));
    appendmode=0;
endif;

if appendmode NE 1;

  /*******************************
  ** Create new output files
  *******************************/

  "-----------------------------------------------------------";
  "Creating new output files";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  /* k = keyw; */
  "";

  create fhstat = ^ostat with ^statname, 5, 4;
  create fhpara = ^opara with ^para_names, npara, 4;

else;

  /*******************************
  ** Open existing output files
  *******************************/

  "-----------------------------------------------------------";
  "Opening Existing Output Files";
  "Press Key to Continue";
  "-----------------------------------------------------------";

  open fhpara = ^opara for update;
  open fhstat = ^ostat for update;

  rowind = seekr(fhpara,rowsf(fhpara));
  rowind = seekr(fhstat,rowsf(fhpara));

  para    = readr(fhpara,1)';
  statsim = readr(fhstat,1);

  "Number of draws in File" rowind;
  "";

  rowind = seekr(fhpara,0);
  rowind = seekr(fhstat,0);

  cc0 = 0;

  "Parameter   | Last Draw ";
  outmat = para_names'~para;
  let mask[1,2] = 0 1;
  let fmt[2,3] =
               "-*.*s " 10 6
               "*.*lf " 10 4;
  d = printfm(outmat,mask,fmt);

  "";
  "Press Key to Continue";
  "-----------------------------------------------------------";
  /* k = keyw; */
  
endif;

/********************************************************
* Metropolis-Hastings algorithm
********************************************************/

/** initialize PARANEW **/
/** trembles the posterior mode, and posterior is large enough **/
valid = 0;
do until valid;
   paranew = para + cc0*(sigscale*rndn(npara,1));
   paranew = paranew.*pmaskinv + pfix.*pmask;

   {postnew, likenew} = fcn2(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;
endo;

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if (iblock == 1) AND (appendmode NE 1);
      /** save initial value as 1st draw in 1st block **/
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
   elseif (iblock == 1) AND (appendmode == 1);
      j = 1;
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
   else;
      /** don't save initial values **/
      j=1;
   endif;

   /** with-in block loop **/
    do until j > nsim;

      /** propose **/
      paranew = paraold + cc*(sigscale*rndn(npara,1));
      paranew = paranew.*pmaskinv + pfix.*pmask;

      /** evaluate at the candidate **/
      {postnew, likenew} = fcn2(paranew);
      propdens = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                 -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
      propsim[j,1] = propdens;
      proppostsim[j,1] = postnew;

      /** accept/reject rule  **/
      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          postsim[j,1] = postnew;
          likesim[j,1] = likenew;
          parasim[j,.] = paranew';

          /** updates **/
          paraold = paranew;
          postold = postnew;
          likeold = likenew;

      else;

          /** reject proposed jump **/
          likesim[j,1] = likeold;
          postsim[j,1] = postold;
          parasim[j,.] = paraold';
          rej[j] = 1;

      endif;

      /**
      "Block" iblock "of" nblocks;
      "Simulation step" j "of" nsim;
      "Proposal Dens  " propsim[j,1];
      "Parameters     "; parasim[j,.];
      **/

        if _verbose;
            if fmod(j,nsim/10)==0;".";; pause(1e-20); endif;
        endif;

      j = j+1;
    endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Rejection: perct " meanc(rej);
       "Likelihood       " meanc(likesim);
       "Posterior        " meanc(postsim);
       "Parameters       "; meanc(parasim)';
       "";
       pause(1e-20);
   endif;  /** verbose **/

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;           /** block **/

closeall fhstat, fhpara;

#include mhdd.g;


/********************************************************
* PROCEDURE: FCN
* defines the objective function:
* log posterior
*********************************************************/
proc (2) = fcn2(para);
local lnpY,lnprio,lnpost,retcode, parabd_ind1, parabd_ind2, penalty, modelpara;

    parabd_ind1 = para .> _bounds[.,1];         /** lower bounds **/
    parabd_ind2 = para .< _bounds[.,2];         /** upper bounds **/
    parabd_ind1 = delif(parabd_ind1,pmask);
    parabd_ind2 = delif(parabd_ind2,pmask);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);   /** in bounds **/
    
        modelpara = para;
        modelpara = modelpara.*pmaskinv + pfix.*pmask;

        lnpy   = marglh(modelpara,msel,lambda);
        lnpy   = real(lnpy);
        penalty = evalpenalty(modelpara,msel,_sstargets);
        lnprio = priodens(modelpara, pmean, pstdd, pshape);
        lnpost = lnpy + real(lnprio) + penalty;
        
    else;                                                           /** out of bounds **/

        lnpost = -1E6;
        lnpy   = -1E20;

    endif;

retp(lnpost,lnpy);
endp;
