/******************************************************************************
* DSGE_WCINFL.G
* for every draw of DSGE parameters, computes the welfare losses (in steady state) of changing the target inflation relative to a benchamark of 2.5% per year. 
*
* created:  03-SEP-2007
* based on: dsge_ss.g
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 0;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;


/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun  = "1";
nblock = 10000;    /* block size */
nuse   = 10;      /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
//opath  = resupath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun3";
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/** check dimensions **/
para    = pmsv(msel);
{sst, valid}=dsgess(para,msel);
/* {vd,ac} = dsgemom(para,msel); */

n_sst = rows(sst);


   /* Setting parameters for target inflation (lower bound, upper      bound and grid points) */

   lowinfl    = 0.9942;  /* lower target is -2.3% per year */
   highinfl   = 1.0075;  /* upper target is 3% per year    */
   targetinfl = 1.00625; /* reference is 2.5% per year     */
   ngrid      = 41;
   incr       = (highinfl-lowinfl)/(ngrid-1);
   infl       = seqa(lowinfl,incr,ngrid);
   infl       = targetinfl|infl;

/** Initialize Output files **/
owfcosts  = opath $+ "\\wfcosts_draws";
oinfltar  = opath $+ "\\infltarget";

create fhwelfarecosts = ^owfcosts with var, ngrid, 8;

create fhinflation = ^oinfltar with var, 1, 8;

/** Initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
   "";
   "Calculating welfare costs...";
   "----------------------------";
endif;

do until eofloop;

   if _verbose;
      "";
      "Block           " ftos(loopct,"%*.*lf",6,0);
   endif;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);



if ismiss(parasim) == 0;

   sstsim   = zeros(nread, n_sst);
   /* acsim = zeros(nread,nac);
   vdsim    = zeros(nread,nvd); */

   welfareloss       = zeros(nread,ngrid+1);
   x                 = zeros(6,ngrid+1);

   j = 1;
   
   do until j > nread;
      para = parasim[j,.]';
            
      {sst, valid} = dsgess(para,msel);
       k=1;
       do until k>ngrid+1;

del    = para[1];         
alp    = para[2];          
lam    = para[3];
gam    = para[4];          
chist  = para[5];
zet    = para[6];
gst    = para[7];
FF     = para[8];
lnmyp  = para[9];
nu     = para[10];
iota   = para[11];

/* fix nu to 3 */
nu = 3;

rr_steady    = para[19];

bet    = 1/(1+rr_steady/400);

pi_steady_st = para[20];

pistst = 1 + pi_steady_st/400;

Rkst   = 1/bet + del - 1;

pist   = infl[k];

A      = sst[21];

B      = sst[23];

Zst    = sst[22];

Rst    = pist/bet;

temp_p = 1/(1-zet) - zet/(1-zet)*(pistst/pist)^(-(1-iota)/lam);

post   = ( temp_p )^(-lam);

AcalF1 = zet*bet*(pistst/pist)^(-(1-iota)/lam);

AcalF2 = zet*bet*(pistst/pist)^(-(1-iota)*(1+lam)/lam);

Dst    = (1-zet)*(post)^(-(1+lam)/lam) / ( 1 - AcalF2/bet );

start  = {0.67,0.035084,0.69,11.5348,17,1};

{x1,tcode}  = eqSolve(&fr,start);

x[.,k] = x1;
k=k+1;
endo;

k=1;

do until k>ngrid+1;

temp_1 = B*ln(x[1,1]) - A*(x[2,1]-x[2,k]);
temp_2 = ((chist/(1-nu))*(A/Zst^(1/(1-alp)))^(1-nu)) * (x[3,1]^(1-nu) - x[3,k]^(1-nu));

welfareloss[j,k] = (exp((1/B)*(temp_1+temp_2))-x[1,k])/x[1,k];

k=k+1;
endo;

      /* Steady states */
      sstsim[j,.] =  sst';

      if _verbose;
         if fmod(j,nread/10)==0; ".";; pause(1e-20);endif;
      endif;
      j = j+1;
   endo;

   welfareloss     =  welfareloss[1:nread,2:ngrid+1];
   inflationtarget = (infl[2:ngrid+1]-1)*400;
   
   
      wr       = writer(fhwelfarecosts,welfareloss);

      wr       = writer(fhinflation,inflationtarget);

   
endif;

   if _verbose;
      "";
      "Draws Processed " ftos(rows(sstsim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhwelfarecosts, fhinflation;

end;

/********************************************************************
**          Procedure to solve for New Steady State
********************************************************************/


proc fr(x);

local f1,f2,f3,f4,f5,f6;

f1 = (Dst*x[6] + FF)/((x[4]^(alp))*x[2]^(1-alp)) - Zst;

f2 = (alp*(x[6]*Dst + FF)*post/((1+lam)*( (1-AcalF1)/(1-AcalF2) )*Rkst)) - x[4];

f3 = (((1-alp)/alp)*x[4]*Rkst/x[2]) - x[5];

f4 = x[6] - del*x[4] - (1-1/gst)*x[6] - x[1];

f5 = (1/A)*((chist*(pist^nu)*x[5]/((Rst-1)*Zst^((1-nu)/(1-alp)) ))^(1/nu)) - x[3];

f6 = B*x[5] - A*x[1]^gam;

retp(f1|f2|f3|f4|f5|f6);

     endp;

