/* filename:    dsge_paratrhpd.g
** description: Computes HPD set via kernel density set 
**              based on the output of the posterior simulator
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;

mhrun1  = "1";    
mhrun2  = "2";

nburn   = 1;        /* Number of initial draws to be discarded */
nuse    = 1;
nblock  = 1000;
covprob = 0.90;


/********************************************************
*  Create a grid for the th1 and th2 (NK_slope, NK_lag)
********************************************************/

/* use this grid for datasel = 1; */
 
th1_min = 0; th1_max = 2.0; th1_n = 101;
th1_grid = seqa(th1_min,(th1_max-th1_min)/(th1_n-1),th1_n);

th2_min = 0; th2_max = 0.5; th2_n = 101;
th2_grid = seqa(th2_min,(th2_max-th2_min)/(th2_n-1),th2_n);


/********************************************************
*  Load first parameter draws (for density estimate)
********************************************************/

"Loading Draws for Density Estimates";
pause(1e-5);

lpath  = resupath $+ "\\mhrun" $+ mhrun1;
ldraw  = lpath $+ "\\paratr";

open fhdraw = ^ldraw for read;

/** Initialization **/
drawrow = seekr( fhdraw, nburn);
eofloop = 0;

drawvec1 = readr( fhdraw,1);
drawvec1 = drawvec1[.,1:2];

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr( fhdraw, nblock );
   nread   = rows( parasim );

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   if ismiss(parasim) == 0;
      drawvec1 = drawvec1 | parasim[.,1:2];
   endif;

   eofloop = eof(fhdraw);

endo;
closeall fhdraw;

drawstdd1 = stdc(drawvec1);
/* h = 0.2*rows(drawvec1)^(-1/6)*drawstdd1; */
h = rows(drawvec1)^(-1/6)*drawstdd1;

/********************************************************
*  load second parameter draws (to evaluate density)
********************************************************/

"Evaluating Density";
pause(1e-5);

lpath  = resupath $+ "\\mhrun" $+ mhrun2;
ldraw  = lpath $+ "\\paratr";

open fhdraw = ^ldraw for read;

/** Initialization **/
drawrow = seekr( fhdraw, nburn);
eofloop = 0;
loopct  = 0;

densvec = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr( fhdraw, nblock );
   nread   = rows( parasim );

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   loopct = loopct + nread;

   if ismiss(parasim) == 0;

      densvec = densvec | densest(parasim[.,1:2],drawvec1,h);
      "Draws processed:" loopct;
      pause(1e-5);   

   endif;

   eofloop = eof(fhdraw);

endo;
closeall fhdraw;

densvec     = sortc(densvec[2:rows(densvec)],1);
covprobindx = round( (1-covprob)*rows(densvec) );
hpdcutoff   = densvec[covprobindx];

/********************************************************
*  construct hpd density set
********************************************************/

/* output in matrix form 
** right-most and bottom column contain grid values
*/
hpd_mat                  = zeros( (th2_n+1), (th1_n+1) );
hpd_mat[1:th2_n,th1_n+1] = th2_grid;
hpd_mat[th2_n+1,1:th1_n] = th1_grid';

/* Evaluate density on grid
** and determine whether parameter belongs to HPD set 
*/

"Determining HPD Set";
pause(1e-5);

ict = 1;
i1  = 1;
do until i1 > th1_n;

   i2 = 1;
   do until i2 > th2_n;

      th12 = th1_grid[i1] ~ th2_grid[i2] ;
      hpd_mat[i2,i1] = ( densest(th12,drawvec1,h) .ge hpdcutoff ) ;

      if fmod(ict,th1_n*th2_n/10)==0; 
          /* ".";; */
          "Step" ict "out of" th1_n*th2_n; 
          pause(1e-20); 
      endif;
              
      i2 = i2 + 1;
      ict = ict+1;
       
   endo;
   
   i1 = i1+1;
   
endo;

/********************************************************
* Derive boundaries of HPD set
********************************************************/

HPDnogrid_mat = HPD_mat[1:th2_n,1:th1_n];

HPD_matsel = ( sumc(HPDnogrid_mat) .== 0 ); 
HPDtr_mat  = ( delif(HPDnogrid_mat',HPD_matsel) )';
HPDth1_grid   = ( delif(th1_grid,HPD_matsel) );

HPD_mat_temp = seqa(1,1,th2_n) .* HPDtr_mat;
HPD_ub = th2_grid[maxindc(HPD_mat_temp[1:th2_n,.])'];

HPD_mat_temp = seqa(th2_n,-1,th2_n) .* HPDtr_mat;
HPD_lb = th2_grid[maxindc(HPD_mat_temp[1:th2_n,.])'];

/* check whether there are open ends */

if HPD_ub[1] ne HPD_lb[1];
   HPDth1_grid = (HPDth1_grid[1] - (HPDth1_grid[2]-HPDth1_grid[1])/100) |
                 HPDth1_grid;
   HPD_diff = (HPD_ub[1]-HPD_lb[1])/2;
   HPD_ub   = (HPD_lb[1]+HPD_diff) | HPD_ub;
   HPD_lb   = (HPD_lb[1]+HPD_diff) | HPD_lb;
endif;

HPDth1_n = rows(HPDth1_grid);

if HPD_ub[HPDth1_n] ne HPD_lb[HPDth1_n];
   HPDth1_grid = HPDth1_grid |
                (HPDth1_grid[HPDth1_n] + (HPDth1_grid[HPDth1_n]-HPDth1_grid[HPDth1_n-1])/100);
   HPD_diff = (HPD_ub[HPDth1_n]-HPD_lb[HPDth1_n])/2;
   HPD_ub   = HPD_ub | (HPD_lb[HPDth1_n]+HPD_diff);
   HPD_lb   = HPD_lb | (HPD_lb[HPDth1_n]+HPD_diff);
endif;


/********************************************************
* Initialize Output files
********************************************************/

opath    = resupath $+ "\\mhrun" $+ mhrun1;
chkdir   = chk_dir(opath);
ohpd     = opath  $+ "\\paratrHPD2dim" $+ ftos(100*covprob,"%*.*lf",2,0);

create fhhpd = ^ohpd with HPD, rows(HPDth1_grid), 8;
wr = writer(fhhpd,(HPDth1_grid~HPD_ub~HPD_lb)');
closeall fhhpd;


/********************************************************
* Plot some results
********************************************************/

graphset;
begwind;
margin(0.2,0.2,0.2,0.2); 

window(1,1,0);
fonts("microb");
_plwidth = 8;
_psymsiz = 3;
_plctrl = -1;
_plctrl=0;

xtics(0,1,0.25,1);
ytics(0,0.5,0.25,1);

_pcolor = {1 1};
_pltype = {6 6};

xlabel("NKPC Slope");
ylabel("NKPC Lag  ");
xy(HPDth1_grid,HPD_ub~HPD_lb);

endwind;

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/


proc (1) = densest(paragrid,draws,h);
/* This procedure computes a 2dim kernel density estimate
** using indep Gaussian kernels
*/
local   ndraws, npt, pdfhat, i;

ndraws = rows(draws);
npt    = rows(paragrid);
pdfhat = zeros(npt,1); 

i = 1;
do until i > npt;
   pdfhat[i] = 1/(h[1]*h[2])
               *meanc(   pdfn( (ones(ndraws,1)*paragrid[i,1]-draws[.,1])/h[1] )
                      .* pdfn( (ones(ndraws,1)*paragrid[i,2]-draws[.,2])/h[2] ) );
   i = i+1;
endo;

retp(pdfhat);
endp;

