/******************************************************************************
* SSPMOM.G
* computes posterior mean and std and CI based on output of posterior simulator
* steady state values of variables
*
* created:  02-SEP-2007
* based on: VDPMOM.G
******************************************************************************/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include dsge_mod.src;
#include dsge_spec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output: Steady States
********************************************************************/
mhrun   = "1";
nburn   = 101;    /* initial draws to be discarded */
hpdprob = 0.90 ;

lpath  = resupath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\paratr";
odraws = opath $+ "\\paratrs";

open fhdraw = ^ldraw for read;

drawrow = readr(fhdraw, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw, nburn);

/********************************************************************
* Part 1: Compute the mean of x(i) and x^2(i)
********************************************************************/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
nblock  = 1000;
ndraws  = 0;

if _verbose;
   "";
   "=============================================";
   "Computing posterior means, stds and CI:     ";
   "Steady state values of variables";
   "";
   "Model            " $lmodel;
   "Data set         " $dataselstr;
   "Prior            " $lprior;
   "Subsample        " $subTstr;
   "MHrun Index      " $mhrun;
   "Coverage         " hpdprob;
   "=============================================";
   "";
   "Part 1: Mean and Standard Deviation";
endif;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);

   drawmean   = drawmean   + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;

   ndraws = ndraws + rows(drawblock);

   if _verbose;
      ".";;pause(1e-20);
   endif;

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = real(sqrt(drawsqmean - (drawmean)^2));
drawcov    = real(drawcross/ndraws - drawmean'*drawmean);

/* Set names of transformed parameters */
draw_names = "PC_slope" $~ "PC_lag" $~ "MD_elas" $~ "NK_distort";
            

if _verbose;
   /** Report Posterior Mean and stderror **/
   "";
   "Parameter        |           Mean |           StdD |";
   outmat = draw_names' $~ ftocv((drawmean' ~ drawstdd'),5,5) ;
   outmat;
   /*   "Parameter | Mean  |  StdD |";
   outmat = draw_names'~ drawmean' ~ drawstdd' ;
   let mask[1,3] = 0 1 1;
   let fmt[3,3] =
                  "-*.*s " 8 6
                  "*.*lf " 8 4
                  "*.*lf " 8 4;

   d = printfm(outmat,mask,fmt);   */
endif;


/********************************************************************
* Part 2: HPD Interval
********************************************************************/
drawrow = seekr( fhdraw, nburn);
drawci  = zeros(2,drawdim);

if _verbose;
   "";
   "Part 2: HPD Interval";
endif;

j = 1;
do until j > drawdim;

   /** Read only the j'th column **/
   drawrow   = seekr(fhdraw,nburn);
   drawblock = readr(fhdraw,1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;

   do until eofloop;

      drawblock = readr(fhdraw,nblock);
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);

   endo;

   if _verbose;".";;pause(1e-20);endif;

   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

/** Report **/
if _verbose;
   "";"";
   "Parameter          |         Mean |           StdD |        CI(LOW) |       CI(HIGH)";
   outmat = draw_names' $~ ftocv( (drawmean' ~ drawstdd' ~ drawci'), 5,5) ;
   outmat;
   /* "Parameter | Mean  |  StdD | CI(LOW) | CI(HIGH)";
   outmat = draw_names'~ drawmean' ~ drawstdd' ~ drawci' ;
   let mask[1,5] = 0 1 1 1 1;
   let fmt[5,3] =
                  "-*.*s " 8 6
                  "*.*lf " 8 4
                  "*.*lf " 8 4   
                  "*.*lf " 8 4
                  "*.*lf " 8 4;

   d = printfm(outmat,mask,fmt); */
   "";
endif;

create fhdraws = ^odraws with SSTSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws;

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim = cols(draws);
ndraws  = rows(draws);
hpdband = zeros(2,drawdim);
nwidth  = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;

retp(hpdband);
endp;
