/******************************************************************/
/* filename :    vm_dummy.g                                       */
/* Description:  Generates dummy observations                     */
/*               for a Minnesota Prior                            */
/******************************************************************/


new;
closeall; 
library pgraph;
cls;

#include vm_spec.g;
#include pathspec.g;

/******************************************************** 
* Import data series, split into YY1 and YY2            *
********************************************************/

#include vm_loaddata.g;

nv  = cols(YY);
nv2 = 1;
nv1 = nv - nv2; 
nex = 2;

YY1 = YY[.,1:nv1];
YY2 = YY[.,nv1+1:nv]; 

nobs    =   rows(YY)-T0;  /* number of observations */
nlag    =   nlags_;       /*lag length*/ 

/******************************************************** 
* Dummy Observations                                    *
********************************************************/

/** Obtain mean and standard deviation 
**  from expandend pre-sample data
**/ 
YY0     =   YY[1:T0+16,.];  
ybar    =   meanc(YY0)';      
sbar    =   stdc(YY0)'; 
premom  =   ybar | sbar;

/** Generate Matrices with Dummy Observations
**/ 
hyp = tau|d|w|lambda|mu; 
{YY1dum,XXdum,breakss} = varprior(nv,nv1,nlag,nex,hyp,premom); 

/** Dummy observations for target inflation 
**/
sbar   = stdc(YY[2:T0+16,nv]-YY[1:T0+15,nv]);
YY2dum = ones(w,1)*sbar;

/******************************************************** 
* Actual Observations                                   *
********************************************************/

YY1act  =   YY1[T0+1:nobs+T0,.]; 
XXact   =   zeros(nobs,nv*nlag); 

i = 1;
do until i > nlag;
   XXact[.,(i-1)*nv+1:i*nv] = YY[T0-(i-1):T0+nobs-i,.];
   i = i+1;
endo;

/** 2nd to last column of XXact = YY2-YY2lag 
**  last column of XXact = constant 
**/
XXact = XXact ~ (YY2[T0+1:nobs+T0,.]-YY2[T0:nobs+T0-1,.])
              ~ ones(nobs,1);  

/** YY2act is inflation differential
**/
YY2act = YY2[T0+1:nobs+T0,.]-YY2[T0:nobs+T0-1,.];

/******************************************************** 
* Store Observations                                    *
********************************************************/

odumobs = priopath $+ "\\dumobs1" $+ ".dat";
create odumobs = ^odumobs with YY1XX, nv1+nv*nlag+nex,8;  
wr = writer(odumobs, ( (YY1dum~XXdum) | (YY1act~XXact)) );
closeall dumobs;

odumobs = priopath $+ "\\dumobs2" $+ ".dat";
create odumobs = ^odumobs with YY2, nv2,8;  
wr = writer(odumobs, ( YY2dum | YY2act ) );
closeall dumobs;

end;

/***************************************************************
The procedure for generating dummy observations  
****************************************************************/ 

proc(3) = varprior(nv,nv1,nlag,nex,hyp,premom); 
/* nv: number of variables
** nlag: number of lags
** nex: number of exogenous variables including intercept
** hyp: vector of hyperparameters
** premom: presample moments - mean | stdd
** XX = [ lags ] [pi*] [constant]
*/

local tau, dd, ww, lam, mm, xdu, ydu, yxdu,
      breakss, dsize, j, sig,lammean, mumean, 
      lamall, muall;   

tau = hyp[1,.];  
dd  = hyp[2,.]; 
ww  = hyp[3,.]; 
lam = hyp[4,.]; 
mm  = hyp[5,.]; 

/* initializations */
dsize   =   nex+(nlag+ww+1)*nv;       
breakss =   zeros(6,1); 
ydu     =   zeros(dsize,nv); 
xdu     =   zeros(dsize,nv*nlag+nex); 

/* Dummies for the coefficients of the first lag
*/ 
sig          =   diagrv(zeros(nv,nv),premom[2,.]');  
ydu[1:nv,.]  =   tau*sig;
xdu[1:nv,.]  =   (tau*sig)~zeros(nv,(nlag-1)*nv+nex);
breakss[1]   =   nv; 

/* Dummies for the coefficients of the remaining lags 
*/
if nlag > 1;
   ydu[breakss[1]+1:nv*nlag,.] = zeros((nlag-1)*nv,nv);
   j = 1;
   do until j > nlag - 1; 
     xdu[breakss[1]+(j-1)*nv+1:breakss[1]+j*nv,.] 
         = zeros(nv,j*nv) ~ tau*sig*((j+1)^dd) ~ zeros(nv,(nlag-1-j)*nv+nex);  
     j = j+1; 
   endo; 
   breakss[2] = breakss[1]+(nlag-1)*nv;   
else;
   breakss[2]=breakss[1]; 
endif;

/* Dummies for the covariance matrix of error terms 
*/ 
ydu[breakss[2]+1:breakss[2]+ww*nv,.] = ones(ww,1).*.sig; 
breakss[3] = breakss[2]+ww*nv; 

/* Dummies for the coefficients of the constant term
*/ 
lammean             = lam*premom[1,.];
ydu[breakss[3]+1,.] = lammean; 
xdu[breakss[3]+1,.] = (ones(1,nlag) .*. lammean) ~ 0 ~ lam; 
breakss[4]          = breakss[3]+1; 

/* Dummies for the coefficients on the effect of target changes
*/ 
xdu[breakss[4]+1,.] = zeros(1,nv*nlag) ~ lam ~ 0;
breakss[5] = breakss[4]+1;

/* Dummies for the covariance matrix of coefficients of different lags 
*/ 
mumean = diagrv(zeros(nv,nv),(mm*premom[1,.])');
ydu[breakss[5]+1:breakss[5]+nv,.]
  = mumean; 
xdu[breakss[5]+1:breakss[5]+nv,.]  
  = (ones(1,nlag) .*. mumean) ~ zeros(nv,nex);
breakss[6] = breakss[5]+nv; 

/* remove last column of ydu and eliminate rows with all zeros
*/
ydu  = ydu[.,1:nv1];
yxdu = ydu~xdu;
yxdu = delif(yxdu,sumc((yxdu .ne 0)').==0);
ydu  = yxdu[.,1:nv1];
xdu  = yxdu[.,nv1+1:cols(yxdu)];

retp(ydu,xdu,breakss); 
endp; 
